package com.blazinggames.tmod.common.map
{
	/**
	 * Your generic interface to tile maps. This is set up as an interface so
	 * that a game with more complex tile data needs or other inheritence 
	 * requirements do not need to use the BasicMap class but can still take
	 * advantage of viewers and other map classes. 
	 * 
	 * Maps are made up of tiles which are simply a number indicating what type
	 * of tile is at that location and attributes which is an optional object
	 * containing tile state information.
	 * 
	 * @author Billy D. Spelchan
	 */
	public interface ITileMap
	{
		/**
		 * Returns the current width of this map (in tiles).
		 * 
		 * @return the current width of the map 
		 */		
		function getWidth():int;
		
		
		/**
		 * Returns the current height of this map (in tiles).
		 * 
		 * @return the current height of the map 
		 */		
		function getHeight():int;
		
		/**
		 * Gets the tile at the indicated location. If the location specified
		 * is outside the map, the coordinates will wrap.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @return the tile at the indicated location.
		 */		
		function getTile(x:int, y:int):int;
		
		/**
		 * Sets the indicated tile to indicated value. If coordinates 
		 * specified are out of bounds, the request is IGNORED! While this
		 * could be wrapped, I feel too much danger of accidental wrapping.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @param value value to put into map
		 */		
		function setTile(x:int, y:int, value:int, attribute:Object = null):void;

		/**
		 * Gets the tile attibutes at the indicated location. If the location specified
		 * is outside the map, the coordinates will wrap.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @return an object containing the tile at the indicated location. Null if no attributes.
		 */		
		function getAttributes(x:int, y:int):Object;
		
		/**
		 * Sets the indicated tile attributes to indicated value. If coordinates 
		 * specified are out of bounds, the request is IGNORED! While this
		 * could be wrapped, I feel too much danger of accidental wrapping.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @param value value to put into attributes
		 */		
		function setAttribute(x:int, y:int, value:Object):void;

	}
}