package com.blazinggames.tmod.common.map
{
	/**
	 * The basic implementation of the ITileMap interface for general purpose 
	 * map use.
	 * 
	 * @author Billy D. Spelchan
	 */
	public class SimpleTileMap implements ITileMap
	{
// ***** VARIABLES *****
		
		private var _width:int;
		private var _height:int;
		protected var _mapData:Array;
		protected var _attributeData:Array;
		protected var _mapChanged:Boolean;
		
// ***** CONSTRUCTOR *****

		public function SimpleTileMap(w:int = 64, h:int = 64, clearID:int = 0)
		{
			var cntrRow:int, cntrCol:int;
			
			_width = w;
			_height = h;
			_mapData = new Array(_height);
			_attributeData = new Array(_height);
			for (cntrRow = 0; cntrRow < _height; ++cntrRow) {
				_mapData[cntrRow] = new Array(_width);
				_attributeData[cntrRow] = new Array(_width);
				for (cntrCol = 0; cntrCol < _width; ++cntrCol) {
					_mapData[cntrRow][cntrCol] = clearID;
					_attributeData[cntrRow][cntrCol] = null;
				}
			}
		}
		
// ***** IMPLEMENTATION OF ITileMap INTERFACE *****
		
		/**
		 * Returns the current width of this map (in tiles).
		 * 
		 * @return the current width of the map 
		 */		
		public function getWidth():int
		{
			return _width;
		}
		
		/**
		 * Returns the current height of this map (in tiles).
		 * 
		 * @return the current height of the map 
		 */		
		public function getHeight():int
		{
			return _height;
		}
		
		/**
		 * Gets the tile at the indicated location. If the location specified
		 * is outside the map, the coordinates will wrap.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @return the tile at the indicated location.
		 */		
		public function getTile(x:int, y:int):int
		{
			var tx:int = x % _width;
			if (tx < 0) tx += _width;
			var ty:int = y % _height;
			if (ty < 0) ty += _height;
			
			return _mapData[ty][tx];
		}
		
		/**
		 * Sets the indicated tile to indicated value. If coordinates 
		 * specified are out of bounds, the request is IGNORED! While this
		 * could be wrapped, I feel too much danger of accidental wrapping.
		 * 
		 * @param x map x coordinate
		 * @param y map y coordinate
		 * @param value value to put into map
		 */		
		public function setTile(x:int, y:int, value:int, attribute:Object = null):void
		{
			if ((x<0) || (y<0) || (x>_width) ||(y>_height)) {
				throw new Error("INVALID COORDINATES IN TILEMAP (SimpleTileMap::setTile("+x+","+y+","+value+")");
				return;
			}
			_mapData[y][x] = value;
			_attributeData[y][x] = attribute;
			_mapChanged = true;
		}
		
		public function getAttributes(x:int, y:int):Object
		{
			var tx:int = x % _width;
			if (tx < 0) tx += _width;
			var ty:int = y % _height;
			if (ty < 0) ty += _height;
			
			return _attributeData[ty][tx];
		}
		
		public function setAttribute(x:int, y:int, value:Object):void
		{
			if ((x<0) || (y<0) || (x>_width) ||(y>_height)) {
				throw new Error("INVALID COORDINATES IN TILEMAP (SimpleTileMap::setTile("+x+","+y+","+value+")");
				return 0;
			}
			_attributeData[y][x] = value;
		}
		
		public function hasChanged():Boolean
		{
			return _mapChanged;
		}
		
		public function changesNoted():void
		{
			_mapChanged = false;
		}
		
// ***** METHODS: Utility functions *****
			
			/**
			 * Clears the map with the indicated value.
			 * 
			 * @param value value to clear the map with.
			 */
			public function clearMap(value:int):void
			{
				var cntrRow:int, cntrCol:int;
				
				for (cntrRow = 0; cntrRow < _height; ++cntrRow)
					for (cntrCol = 0; cntrCol < _width; ++cntrCol)
						_mapData[cntrRow][cntrCol] = value;
				_mapChanged = true;
			}
		
			public function setTilesFromMapData(data:Array, width:int, height:int, offsetX:int = 0, offsetY:int = 0)
			{
				if ((offsetX < 0) || (offsetY < 0))
					return;
				var w = Math.min(width, _width - offsetX);
				var h = Math.min(height, _height - offsetY);
				var indx:int = 0;
				for (var cntrY:int = 0; cntrY < h; ++cntrY) {
					for (var cntrX:int = 0; cntrX < w; ++cntrX) {
						_mapData[cntrY + offsetY][cntrX + offsetX] = data[indx];
						++indx;
					}
				}
				_mapChanged = true;
			}
			/* TODO
			public function copy(src:ITileMap, x:int, y:int, w:int, h:int):void
			{
			}
			*/
			
			public function createClip(x:int, y:int, w:int, h:int):SimpleTileMap
			{
				var clip:SimpleTileMap = new SimpleTileMap(w,h);
				for (var cntrRow:int = 0; cntrRow < h; ++cntrRow)
					for (var cntrCol:int = 0; cntrCol < w; ++cntrCol)
						clip.setTile(cntrCol, cntrRow, getTile(x + cntrCol, y + cntrRow));
				return clip;
			}
			/* TODO
			public function fillRect(x:int, y:int, w:int, h:int, value:int):void
			{
			
			}
			*/

	}
}